import 'dart:async';

import 'package:webroker/client.dart' as webroker;
import 'package:webroker/target.dart';

import 'addr.dart';
import 'node.dart';

abstract class EventHandler {
  Future<void> onConnect(Target webroker);

  Future<void> onError(Object error);

  Future<void> onDisconnect(Target webroker, Object? error);
}

class Listener {
  final EventHandler _eh;

  List<Addr> brokers;
  Node node;
  bool _close = false;

  Listener(this.brokers, this.node, this._eh);

  close() {
    _close = true;
  }

  Future<void> _connect(Addr broker) async {
    var client = await webroker.Client.connect(broker.host, broker.port);
    var target = Target(client.host, client.port, client.id);
    try {
      node.peer.webroker = target;
      await node.publish();
      if (!_close) await _eh.onConnect(target);
      while (!_close) {
        var msg = await client.next();
        try {
          await node.input(msg);
        } catch (e) {
          _eh.onError(e);
        }
      }
      node.peer.webroker = null;
      await _eh.onDisconnect(target, null);
    } catch (e) {
      node.peer.webroker = null;
      await _eh.onDisconnect(target, e);
      rethrow;
    } finally {
      node.peer.webroker = null;
      try {
        await client.close();
      } catch (e) {
        await _eh.onError(e);
      }
    }
  }

  Future<void> listen() async {
    for (int i = 0; _close != true; ++i) {
      var broker = brokers[i % brokers.length];
      try {
        await _connect(broker);
        await Future.delayed(Duration(milliseconds: 200));
      } catch (e) {
        await _eh.onError(e);
      }
    }
  }
}
