import 'package:base32/base32.dart';
import 'package:webroker/target.dart';
import 'package:yona_crypto/b64.dart';

import 'addr.dart';

class Peer {
  final Addr verival;
  final B64 pk;
  Target? webroker;

  @override
  bool operator ==(Object other) => other is Peer && other.pk == pk;

  Peer(this.pk, this.verival, [this.webroker]);

  @override
  String toString() {
    List<String> path = [];
    if (webroker != null) {
      path = [Addr(webroker!.host, webroker!.port).toString(), webroker!.id];
    }
    return Uri(
            scheme: "yona",
            host: base32.encode(pk.decode()),
            userInfo: verival.toString(),
            pathSegments: path)
        .toString();
  }

  static Peer fromString(String peer) {
    var uri = Uri.parse(peer);
    if (uri.scheme != "yona") {
      throw Exception("invalid uri scheme");
    }
    var verival = Addr.fromString(uri.userInfo);
    Target? webroker;
    if (uri.pathSegments.isNotEmpty) {
      var webrokerAddr = Addr.fromString(uri.pathSegments[0]);
      webroker =
          Target(webrokerAddr.host, webrokerAddr.port, uri.pathSegments[1]);
    }
    return Peer(
        B64.encode(base32.decode(uri.host.toUpperCase())), verival, webroker);
  }

  @override
  int get hashCode => Object.hash(pk, verival, webroker);
}
