import 'dart:async';
import 'dart:convert';

import 'package:webinbox/webinbox.dart' as webinbox;

import 'addr.dart';
import 'node.dart';

abstract class EventHandler {
  Future<void> onError(Object error);

  Future<String?> getLast();

  Future<void> setLast(String since);
}

class Poller {
  final EventHandler _eh;

  Addr addr;
  Node node;
  bool _close = false;

  Poller(this.addr, this.node, this._eh);

  close() {
    _close = true;
  }

  Future<void> _poll() async {
    var msg = await webinbox.get(
        Uri(host: node.webinbox.host, port: node.webinbox.port, scheme: "http"),
        node.peer.pk,
        await _eh.getLast());
    var out = Uri(queryParameters: {
      "from": msg.from,
      "to": msg.to,
      "cipher": msg.data,
    }).query;
    await node.input(utf8.encode(out));
    await _eh.setLast(msg.id!);
  }

  Future<void> start() async {
    while (!_close) {
      try {
        await _poll();
      } catch (e) {
        await _eh.onError(e);
      }
    }
  }
}
